package com.bsoft.api.service.Impl;

import com.bsoft.api.common.enums.ProjectType;
import com.bsoft.api.common.enums.StateType;
import com.bsoft.api.common.enums.TypeState;
import com.bsoft.api.mapper.SerPageProjValueMapper;
import com.bsoft.api.mapper.SerPageValueConfigMapper;
import com.bsoft.api.mapper.SerProjMapper;
import com.bsoft.api.mapper.SerProjValueMapper;
import com.bsoft.api.model.SerPageValueConfig;
import com.bsoft.api.model.SerProjValue;
import com.bsoft.api.model.reqmodel.AddBudgetValue;
import com.bsoft.api.model.reqmodel.BudgetValue;
import com.bsoft.api.model.respmodel.*;
import com.bsoft.api.service.SerProjValueService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;

@Service
public class SerProjValueServiceImpl implements SerProjValueService {
    @Resource
    private SerProjMapper serProjMapper;
    @Resource
    private SerProjValueMapper serProjValueMapper;
    @Resource
    private SerPageProjValueMapper pageProjValueMapper;
    @Resource
    private SerPageValueConfigMapper pageValueConfigMapper;

    //查询项目数值
    @Override
    public Object getValue(Integer projectType,Integer date,String deptCode) throws Throwable {
        List<ProjectValue> result = new ArrayList<>();
        List<Map<String,Object>> enumList = TypeState.all();
        if(enumList != null && enumList.size() > 0){
            ProjectValue projectValue = null;
            List<ProjectInfo> projectInfo = null;
            for(Map<String,Object> enumInfo : enumList){
                projectValue = new ProjectValue();
                projectValue.setTypeId((Integer)enumInfo.get("value"));
                projectInfo = new ArrayList<>();
                projectInfo = serProjMapper.selectState(projectType,(Integer)enumInfo.get("value"));
                if(projectInfo != null && projectInfo.size() > 0){
                    for(ProjectInfo projInfo : projectInfo){
                        List<SerProjValueResp> values = serProjValueMapper.selectValue(projectType,(Integer)enumInfo.get("value"),
                                projInfo.getExponentId(),deptCode,date,null);
                        if(values != null && values.size() > 0)
                            projInfo.setProjectValues(values);
                    }
                }
                projectValue.setProject(projectInfo);
                result.add(projectValue);
            }
        }
        return result;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean save(AddBudgetValue request) {
        int result = 0;
        if(request != null && request.getBudgetValues() != null && request.getBudgetValues().size() > 0){
            SerProjValue info = null;
            long version = System.currentTimeMillis();
            for(BudgetValue budgetValue : request.getBudgetValues()){
                info = new SerProjValue(budgetValue);
                info.setDate(request.getDate());
                info.setDeptCode(request.getDeptCode());
                info.setCreateDate(new Date());
                info.setState(BigDecimal.valueOf(StateType.ON.getValue()));
                info.setVersion(String.valueOf(version));
                result = serProjValueMapper.insert(info);
            }
        }
        return result > 0;
    }

    @Override
    public Object getSummary(Integer date) {
        List<Summary> result = serProjMapper.selectDept(date);
        if(result != null && result.size() > 0){
            for(Summary summary : result){
                summary.setProjTypeList(new ArrayList<>());
                List<Map<String,Object>> enumList = ProjectType.all();
                SummaryProjType projType = null;
                List<ProjectInfo> projectInfo = null;
                if(enumList != null && enumList.size() > 0){
                    for(Map<String,Object> enumInfo : enumList){
                        projType = new SummaryProjType();
                        projType.setProjTypeName(String.valueOf(enumInfo.get("description")));
                        List<SerProjValueResp> values = serProjValueMapper.selectValue((Integer)enumInfo.get("value"),null,
                                null,summary.getDeptCode(),date,1);
                        if(values != null)
                            projType.setProject(values);
                        summary.getProjTypeList().add(projType);
                    }
                }else{
                    summary.setProjTypeList(null);
                }
            }
        }
        return result;
    }

    @Override
    public Object getData(Integer date,Integer page) {
        List<SummaryNew> result = pageProjValueMapper.selectDept(date,page);
        if(result != null && result.size() > 0){
            for(SummaryNew summary : result){
                List<SummaryValue> values = pageProjValueMapper.selectValue(date,page,summary.getDeptCode());
                summary.setProjTypeList(values);
            }
        }
        return result;
    }

    @Override
    public Object getDataNew(Integer page,Map<String,String> dim) {
        SerPageValueConfig pageValueConfig = pageValueConfigMapper.selectByPage(page);
        if(pageValueConfig != null){
            Map<String,String> map = new HashMap<String,String>() {
                {
                    put("sqlStr",pageValueConfig.getSqlstr());
                    putAll(dim);
                }
            };
            List<Map<String,Object>> dataList = pageValueConfigMapper.selectSqlStr(map);
            return dataList;
        }
        return null;
    }


}
